# coding: utf-8

"""
    OpenShift API (with Kubernetes)

    OpenShift provides builds, application lifecycle, image content management, and administrative policy on top of Kubernetes. The API allows consistent management of those objects.  All API operations are authenticated via an Authorization bearer token that is provided for service accounts as a generated secret (in JWT form) or via the native OAuth endpoint located at /oauth/authorize. Core infrastructure components may use client certificates that require no authentication.  All API operations return a 'resourceVersion' string that represents the version of the object in the underlying storage. The standard LIST operation performs a snapshot read of the underlying objects, returning a resourceVersion representing a consistent version of the listed objects. The WATCH operation allows all updates to a set of objects after the provided resourceVersion to be observed by a client. By listing and beginning a watch from the returned resourceVersion, clients may observe a consistent view of the state of one or more objects. Note that WATCH always returns the update after the provided resourceVersion. Watch may be extended a limited time in the past - using etcd 2 the watch window is 1000 events (which on a large cluster may only be a few tens of seconds) so clients must explicitly handle the \"watch to old error\" by re-listing.  Objects are divided into two rough categories - those that have a lifecycle and must reflect the state of the cluster, and those that have no state. Objects with lifecycle typically have three main sections:  * 'metadata' common to all objects * a 'spec' that represents the desired state * a 'status' that represents how much of the desired state is reflected on   the cluster at the current time  Objects that have no state have 'metadata' but may lack a 'spec' or 'status' section.  Objects are divided into those that are namespace scoped (only exist inside of a namespace) and those that are cluster scoped (exist outside of a namespace). A namespace scoped resource will be deleted when the namespace is deleted and cannot be created if the namespace has not yet been created or is in the process of deletion. Cluster scoped resources are typically only accessible to admins - resources like nodes, persistent volumes, and cluster policy.  All objects have a schema that is a combination of the 'kind' and 'apiVersion' fields. This schema is additive only for any given version - no backwards incompatible changes are allowed without incrementing the apiVersion. The server will return and accept a number of standard responses that share a common schema - for instance, the common error type is 'unversioned.Status' (described below) and will be returned on any error from the API server.  The API is available in multiple serialization formats - the default is JSON (Accept: application/json and Content-Type: application/json) but clients may also use YAML (application/yaml) or the native Protobuf schema (application/vnd.kubernetes.protobuf). Note that the format of the WATCH API call is slightly different - for JSON it returns newline delimited objects while for Protobuf it returns length-delimited frames (4 bytes in network-order) that contain a 'versioned.Watch' Protobuf object.  See the OpenShift documentation at https://docs.openshift.org for more information. 

    OpenAPI spec version: latest
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from pprint import pformat
from six import iteritems
import re


class V1ClairReportVulnerability(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, name=None, namespace_name=None, description=None, link=None, severity=None, fixed_by=None):
        """
        V1ClairReportVulnerability - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'name': 'str',
            'namespace_name': 'str',
            'description': 'str',
            'link': 'str',
            'severity': 'str',
            'fixed_by': 'str'
        }

        self.attribute_map = {
            'name': 'name',
            'namespace_name': 'namespaceName',
            'description': 'description',
            'link': 'link',
            'severity': 'severity',
            'fixed_by': 'fixedBy'
        }

        self._name = name
        self._namespace_name = namespace_name
        self._description = description
        self._link = link
        self._severity = severity
        self._fixed_by = fixed_by

    @property
    def name(self):
        """
        Gets the name of this V1ClairReportVulnerability.
        Vulnerability name

        :return: The name of this V1ClairReportVulnerability.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Sets the name of this V1ClairReportVulnerability.
        Vulnerability name

        :param name: The name of this V1ClairReportVulnerability.
        :type: str
        """

        self._name = name

    @property
    def namespace_name(self):
        """
        Gets the namespace_name of this V1ClairReportVulnerability.
        Vulnerability namespace name

        :return: The namespace_name of this V1ClairReportVulnerability.
        :rtype: str
        """
        return self._namespace_name

    @namespace_name.setter
    def namespace_name(self, namespace_name):
        """
        Sets the namespace_name of this V1ClairReportVulnerability.
        Vulnerability namespace name

        :param namespace_name: The namespace_name of this V1ClairReportVulnerability.
        :type: str
        """

        self._namespace_name = namespace_name

    @property
    def description(self):
        """
        Gets the description of this V1ClairReportVulnerability.
        Vulnerability description

        :return: The description of this V1ClairReportVulnerability.
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Sets the description of this V1ClairReportVulnerability.
        Vulnerability description

        :param description: The description of this V1ClairReportVulnerability.
        :type: str
        """

        self._description = description

    @property
    def link(self):
        """
        Gets the link of this V1ClairReportVulnerability.
        Vulnerability link

        :return: The link of this V1ClairReportVulnerability.
        :rtype: str
        """
        return self._link

    @link.setter
    def link(self, link):
        """
        Sets the link of this V1ClairReportVulnerability.
        Vulnerability link

        :param link: The link of this V1ClairReportVulnerability.
        :type: str
        """

        self._link = link

    @property
    def severity(self):
        """
        Gets the severity of this V1ClairReportVulnerability.
        Vulnerability severity

        :return: The severity of this V1ClairReportVulnerability.
        :rtype: str
        """
        return self._severity

    @severity.setter
    def severity(self, severity):
        """
        Sets the severity of this V1ClairReportVulnerability.
        Vulnerability severity

        :param severity: The severity of this V1ClairReportVulnerability.
        :type: str
        """

        self._severity = severity

    @property
    def fixed_by(self):
        """
        Gets the fixed_by of this V1ClairReportVulnerability.
        Which package version this vulnerability was fixed

        :return: The fixed_by of this V1ClairReportVulnerability.
        :rtype: str
        """
        return self._fixed_by

    @fixed_by.setter
    def fixed_by(self, fixed_by):
        """
        Sets the fixed_by of this V1ClairReportVulnerability.
        Which package version this vulnerability was fixed

        :param fixed_by: The fixed_by of this V1ClairReportVulnerability.
        :type: str
        """

        self._fixed_by = fixed_by

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        if not isinstance(other, V1ClairReportVulnerability):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other
